


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v1_00_b;
use proc_common_v1_00_b.proc_common_pkg.all;

entity vector_proc_32 is
 port (
DO: out std_logic_vector(31 downto 0);
ADDR: in std_logic_vector(31 downto 0);
DI: in std_logic_vector(31 downto 0);
EN: in std_logic_vector(0 downto 0);
WE: in std_logic_vector(0 downto 0);
REG_IN_A: in std_logic_vector(31 downto 0);
REG_IN_B: in std_logic_vector(31 downto 0);
REG_OUT_A: out std_logic_vector(31 downto 0);
REG_OUT_B: out std_logic_vector(31 downto 0);
REG_WE_A: in std_logic_vector(0 downto 0);
REG_WE_B: in std_logic_vector(0 downto 0);
Clk: in std_logic;
Reset: in std_logic

);
end vector_proc_32;
architecture IMP of vector_proc_32 is

  component HI_RAMB
   generic (
	data_width: integer;
	address_width: integer;
	depth: integer
	);
   port (
	doa: out std_logic_vector(data_width-1 downto 0);
	dob: out std_logic_vector(data_width-1 downto 0);
	addra: in std_logic_vector(address_width-1 downto 0);
	addrb: in std_logic_vector(address_width-1 downto 0);
	clka: in std_ulogic;
	clkb: in std_ulogic;
	dia: in std_logic_vector(data_width-1 downto 0);
	dib: in std_logic_vector(data_width-1 downto 0);
	rsta: in std_ulogic;
	rstb: in std_ulogic;
	wea: in std_ulogic;
	web: in std_ulogic
	);
  end component;


	signal L: std_logic_vector( 31 downto 0 );
	signal sum: std_logic_vector( 31 downto 0 );
	signal ready: std_logic_vector( 31 downto 0 );
	type l1w32_type is array (0 to 0) of std_logic_vector(31 downto 0);
	type l1w1_type is array (0 to 0) of std_logic_vector(0 downto 0);
	signal array_addra: l1w32_type;
	signal array_addrb: l1w32_type;
	signal array_dina: l1w32_type;
	signal array_doutb: l1w32_type;
	signal array_wea: l1w1_type;
	signal array_dinb: l1w32_type;
	signal array_douta: l1w32_type;
	signal array_web: l1w1_type;

begin

array_gen: for array_i in 0 to 0 generate
begin
	array_inst: HI_RAMB
	generic map (
		 address_width => 32,
		 data_width => 32,
		 depth => 16384
	)
	port map (
		addra => array_addra(array_i),
		addrb => array_addrb(array_i),
		dia => array_dina(array_i),
		wea => array_wea(array_i)(0),
		dob => array_doutb(array_i),
		doa => array_douta(array_i),
		dib => array_dinb(array_i),
		web => array_web(array_i)(0),
		clka => Clk,
		clkb => Clk,
		rsta => Reset,
		rstb => Reset
	);
end generate array_gen;
REG_OUT_A <= sum;
REG_OUT_B <= ready;
array_dina(0) <= DI;
array_addra(0) <= ADDR;
array_wea(0) <= WE;
DO <= array_douta(0);

STATE_PROC: process ( Clk ) is
  variable stage: integer := 0;
begin
 if ( Clk'event and Clk = '1' ) then
  if ( Reset = '1' ) then
ready <= "00000000000000000000000000000000";
sum <= "00000000000000000000000000000000";
array_web(0) <= "0";
else
    case stage is
when 0 =>
  stage := stage + 1;
if (REG_WE_A = "1") then
L <= REG_IN_A;
array_addrb(0) <= "00000000000000000000000000000000";
ready <= "00000000000000000000000000000000";
sum <= "00000000000000000000000000000000";
else
	stage := 0;
end if;
when 1 =>
  stage := stage + 1;
array_addrb(0) <= array_addrb(0) + "00000000000000000000000000000001";
L <= L + "11111111111111111111111111111111";
when 2 =>
  stage := stage + 1;
sum <= sum+array_doutb(0);
array_addrb(0) <= array_addrb(0) + "00000000000000000000000000000001";
L <= L + "11111111111111111111111111111111";
if (L /= "00000000000000000000000000000000") then
	stage := 2;
else
ready <= "00000000000000000000000000000001";
	stage := 0;
end if;
when others => null;
    end case;
  end if;
 end if;
end process STATE_PROC;
end IMP;
