


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v1_00_b;
use proc_common_v1_00_b.proc_common_pkg.all;

entity vector_proc_32 is
 port (
DO: out std_logic_vector(31 downto 0);
ADDR: in std_logic_vector(31 downto 0);
DI: in std_logic_vector(31 downto 0);
EN: in std_logic_vector(0 downto 0);
WE: in std_logic_vector(0 downto 0);
REG_IN_A: in std_logic_vector(31 downto 0);
REG_IN_B: in std_logic_vector(31 downto 0);
REG_OUT_A: out std_logic_vector(31 downto 0);
REG_OUT_B: out std_logic_vector(31 downto 0);
REG_WE_A: in std_logic_vector(0 downto 0);
REG_WE_B: in std_logic_vector(0 downto 0);
Clk: in std_logic;
Reset: in std_logic

);
end vector_proc_32;
architecture IMP of vector_proc_32 is

  component HI_RAMB
   generic (
	data_width: integer;
	address_width: integer;
	depth: integer
	);
   port (
	doa: out std_logic_vector(data_width-1 downto 0);
	dob: out std_logic_vector(data_width-1 downto 0);
	addra: in std_logic_vector(address_width-1 downto 0);
	addrb: in std_logic_vector(address_width-1 downto 0);
	clka: in std_ulogic;
	clkb: in std_ulogic;
	dia: in std_logic_vector(data_width-1 downto 0);
	dib: in std_logic_vector(data_width-1 downto 0);
	rsta: in std_ulogic;
	rstb: in std_ulogic;
	wea: in std_ulogic;
	web: in std_ulogic
	);
  end component;


	signal L: std_logic_vector( 31 downto 0 );
	signal sum: std_logic_vector( 31 downto 0 );
	signal ready: std_logic_vector( 31 downto 0 );
	type l8w32_type is array (0 to 7) of std_logic_vector(31 downto 0);
	type l8w1_type is array (0 to 7) of std_logic_vector(0 downto 0);
	signal array_addra: l8w32_type;
	signal array_addrb: l8w32_type;
	signal array_dina: l8w32_type;
	signal array_doutb: l8w32_type;
	signal array_wea: l8w1_type;
	signal array_dinb: l8w32_type;
	signal array_douta: l8w32_type;
	signal array_web: l8w1_type;
	signal old_addr_low: std_logic_vector( 2 downto 0 );
	signal partsum: l8w32_type;
	signal dout_half: l8w32_type;

begin

array_gen: for array_i in 0 to 7 generate
begin
	array_inst: HI_RAMB
	generic map (
		 address_width => 32,
		 data_width => 32,
		 depth => 2048
	)
	port map (
		addra => array_addra(array_i),
		addrb => array_addrb(array_i),
		dia => array_dina(array_i),
		wea => array_wea(array_i)(0),
		dob => array_doutb(array_i),
		doa => array_douta(array_i),
		dib => array_dinb(array_i),
		web => array_web(array_i)(0),
		clka => Clk,
		clkb => Clk,
		rsta => Reset,
		rstb => Reset
	);
end generate array_gen;
REG_OUT_A <= sum;
REG_OUT_B <= ready;
loop25_gen: for i1 in 0 to 7 generate
begin
array_dina(i1) <= DI;
array_addra(i1)(28 downto 0) <= ADDR(31 downto 3);
array_addra(i1)(31 downto 29) <= "000";
array_wea(i1) <= "1" when ((i1 = ADDR(2 downto 0)) and (WE = "1")) else "0";
DO <= array_douta(i1) when (i1 = old_addr_low) else (others => 'Z');
dout_half(i1)(30 downto 0) <= array_doutb(i1)(31 downto 1);
dout_half(i1)(31 downto 31) <= "0";
end generate loop25_gen;

STATE_PROC: process ( Clk ) is
  variable stage: integer := 0;
begin
 if ( Clk'event and Clk = '1' ) then
  if ( Reset = '1' ) then
ready <= "00000000000000000000000000000000";
sum <= "00000000000000000000000000000000";
for i37 in 0 to 7 loop
partsum(i37) <= "00000000000000000000000000000000";
end loop;
for i38 in 0 to 7 loop
array_web(i38) <= "0";
end loop;
L <= "00000000000000000000000000000000";
else
old_addr_low <= ADDR(2 downto 0);
    case stage is
when 0 =>
  stage := stage + 1;
if (REG_WE_A = "1") then
L(28 downto 0) <= REG_IN_A(31 downto 3);
for i47 in 0 to 7 loop
array_addrb(i47) <= "00000000000000000000000000000000";
end loop;
ready <= "00000000000000000000000000000000";
sum <= "00000000000000000000000000000000";
else
	stage := 0;
end if;
when 1 =>
  stage := stage + 1;
for i55 in 0 to 7 loop
array_addrb(i55) <= array_addrb(i55)+"00000000000000000000000000000001";
end loop;
L <= L+"11111111111111111111111111111111";
when 2 =>
  stage := stage + 1;
for i1 in 0 to 7 loop
if (((i1 = 0) and (array_addrb(0) = "00000000000000000000000000000001")) or ((i1 = 7) and (L = "00000000000000000000000000000000"))) then
partsum(i1) <= partsum(i1)+dout_half(i1);
else
partsum(i1) <= partsum(i1)+array_doutb(i1);
end if;
end loop;
for i67 in 0 to 7 loop
array_addrb(i67) <= array_addrb(i67)+"00000000000000000000000000000001";
end loop;
L <= L+"11111111111111111111111111111111";
if (L /= "00000000000000000000000000000000") then
	stage := 2;
end if;
when 3 =>
  stage := stage + 1;
for i1 in 0 to 3 loop
partsum(i1) <= partsum(i1)+partsum(i1+4);
end loop;
when 4 =>
  stage := stage + 1;
for i1 in 0 to 1 loop
partsum(i1) <= partsum(i1)+partsum(i1+2);
end loop;
when 5 =>
  stage := stage + 1;
sum <= partsum(0)+partsum(1);
ready <= "00000000000000000000000000000001";
	stage := 0;
when others => null;
    end case;
  end if;
 end if;
end process STATE_PROC;
end IMP;
