


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v1_00_b;
use proc_common_v1_00_b.proc_common_pkg.all;

entity vector_proc_32 is
 port (
DO: out std_logic_vector(31 downto 0);
ADDR: in std_logic_vector(31 downto 0);
DI: in std_logic_vector(31 downto 0);
EN: in std_logic_vector(0 downto 0);
WE: in std_logic_vector(0 downto 0);
REG_IN_A: in std_logic_vector(31 downto 0);
REG_IN_B: in std_logic_vector(31 downto 0);
REG_OUT_A: out std_logic_vector(31 downto 0);
REG_OUT_B: out std_logic_vector(31 downto 0);
REG_WE_A: in std_logic_vector(0 downto 0);
REG_WE_B: in std_logic_vector(0 downto 0);
Clk: in std_logic;
Reset: in std_logic

);
end vector_proc_32;
architecture IMP of vector_proc_32 is

  component HI_RAMB
   generic (
	data_width: integer;
	address_width: integer;
	depth: integer
	);
   port (
	doa: out std_logic_vector(data_width-1 downto 0);
	dob: out std_logic_vector(data_width-1 downto 0);
	addra: in std_logic_vector(address_width-1 downto 0);
	addrb: in std_logic_vector(address_width-1 downto 0);
	clka: in std_ulogic;
	clkb: in std_ulogic;
	dia: in std_logic_vector(data_width-1 downto 0);
	dib: in std_logic_vector(data_width-1 downto 0);
	rsta: in std_ulogic;
	rstb: in std_ulogic;
	wea: in std_ulogic;
	web: in std_ulogic
	);
  end component;


	signal L: std_logic_vector( 31 downto 0 );
	signal sum: std_logic_vector( 31 downto 0 );
	signal ready: std_logic_vector( 31 downto 0 );
	type l8w32_type is array (0 to 7) of std_logic_vector(31 downto 0);
	type l8w1_type is array (0 to 7) of std_logic_vector(0 downto 0);
	signal array_addra: l8w32_type;
	signal array_addrb: l8w32_type;
	signal array_dina: l8w32_type;
	signal array_doutb: l8w32_type;
	signal array_wea: l8w1_type;
	signal array_dinb: l8w32_type;
	signal array_douta: l8w32_type;
	signal array_web: l8w1_type;
	signal old_addr_low: std_logic_vector( 2 downto 0 );
	signal partsum: l8w32_type;
	signal summator_in_a: l8w32_type;
	signal summator_in_b: l8w32_type;
	signal summator_in_a_aux: l8w32_type;
	signal summator_in_b_aux: l8w32_type;
	signal summator_out: l8w32_type;
	signal enable_sum: std_logic_vector( 0 downto 0 );
	signal enable_head: std_logic_vector( 0 downto 0 );
	signal enable_tail: std_logic_vector( 0 downto 0 );
	signal cycle: std_logic_vector( 3 downto 0 );

   component floating_add                                                         
	port (                                             
	a             : IN  std_logic_VECTOR(31 downto 0); 
	b             : IN  std_logic_VECTOR(31 downto 0); 
	operation_nd  : IN  std_logic;                                        
	clk           : IN  std_logic;                     
	result        : OUT std_logic_VECTOR(31 downto 0); 
	rdy           : OUT std_logic                      
       );                                                  
   end component;                                          


begin

array_gen: for array_i in 0 to 7 generate
begin
	array_inst: HI_RAMB
	generic map (
		 address_width => 32,
		 data_width => 32,
		 depth => 2048
	)
	port map (
		addra => array_addra(array_i),
		addrb => array_addrb(array_i),
		dia => array_dina(array_i),
		wea => array_wea(array_i)(0),
		dob => array_doutb(array_i),
		doa => array_douta(array_i),
		dib => array_dinb(array_i),
		web => array_web(array_i)(0),
		clka => Clk,
		clkb => Clk,
		rsta => Reset,
		rstb => Reset
	);
end generate array_gen;
REG_OUT_A <= sum;
REG_OUT_B <= ready;
loop34_gen: for i1 in 0 to 7 generate
begin
array_dina(i1) <= DI;
array_addra(i1)(28 downto 0) <= ADDR(31 downto 3);
array_addra(i1)(31 downto 29) <= "000";
array_wea(i1) <= "1" when ((i1 = ADDR(2 downto 0)) and (WE = "1")) else "0";
DO <= array_douta(i1) when (i1 = old_addr_low) else (others => 'Z');
end generate loop34_gen;
loop41_gen: for i1 in 0 to 7 generate
begin

   COMPONENT1: floating_add
	port map (
	a => summator_in_a(i1),
	b => summator_in_b(i1),
	result => summator_out(i1),
	operation_nd => enable_sum(0),
	clk => Clk
	 );
summator_in_a(i1) <= "00000000000000000000000000000000" when (enable_head = "1") else
	summator_in_a_aux(i1) when (enable_tail = "1") else
	 summator_out(i1);
summator_in_b(i1) <= summator_in_b_aux(i1) when (enable_tail = "1") else
	 array_doutb(i1);
end generate loop41_gen;

STATE_PROC: process ( Clk ) is
  variable stage: integer := 0;
begin
 if ( Clk'event and Clk = '1' ) then
  if ( Reset = '1' ) then
ready <= "00000000000000000000000000000000";
sum <= "00000000000000000000000000000000";
for i59 in 0 to 7 loop
array_web(i59) <= "0";
end loop;
enable_sum <= "0";
enable_head <= "1";
enable_tail <= "0";
cycle <= "0000";
L <= "00000000000000000000000000000000";
else
old_addr_low <= ADDR(2 downto 0);
    case stage is
when 0 =>
  stage := stage + 1;
if (REG_WE_A = "1") then
L(28 downto 0) <= REG_IN_A(31 downto 3);
for i72 in 0 to 7 loop
array_addrb(i72) <= "00000000000000000000000000000000";
end loop;
ready <= "00000000000000000000000000000000";
sum <= "00000000000000000000000000000000";
for i75 in 0 to 7 loop
array_web(i75) <= "0";
end loop;
enable_sum <= "0";
enable_head <= "1";
enable_tail <= "0";
cycle <= "0000";
else
	stage := 0;
end if;
when 1 =>
  stage := stage + 1;
for i85 in 0 to 7 loop
array_addrb(i85) <= array_addrb(i85)+"00000000000000000000000000000001";
end loop;
L <= L+"11111111111111111111111111111111";
enable_sum <= "1";
cycle <= "0100";
when 2 =>
  stage := stage + 1;
for i92 in 0 to 7 loop
array_addrb(i92) <= array_addrb(i92)+"00000000000000000000000000000001";
end loop;
cycle <= cycle+"1111";
if (cycle = "0000") then
enable_head <= "0";
end if;
L <= L+"11111111111111111111111111111111";
if (L /= "00000000000000000000000000000000") then
	stage := 2;
else
enable_tail <= "1";
end if;
when 3 =>
  stage := stage + 1;
for i105 in 0 to 7 loop
partsum(i105) <= summator_out(i105);
end loop;
when 4 =>
  stage := stage + 1;
for i108 in 0 to 7 loop
summator_in_a_aux(i108) <= partsum(i108);
end loop;
for i109 in 0 to 7 loop
summator_in_b_aux(i109) <= summator_out(i109);
end loop;
when 5 =>
  stage := stage + 1;
for i112 in 0 to 7 loop
partsum(i112) <= summator_out(i112);
end loop;
when 6 =>
  stage := stage + 1;
for i115 in 0 to 7 loop
summator_in_a_aux(i115) <= partsum(i115);
end loop;
for i116 in 0 to 7 loop
summator_in_b_aux(i116) <= summator_out(i116);
end loop;
when 7 =>
  stage := stage + 1;
for i119 in 0 to 7 loop
partsum(i119) <= summator_out(i119);
end loop;
when 8 =>
  stage := stage + 1;
for i122 in 0 to 7 loop
summator_in_a_aux(i122) <= partsum(i122);
end loop;
for i123 in 0 to 7 loop
summator_in_b_aux(i123) <= summator_out(i123);
end loop;
when 9 =>
  stage := stage + 1;
when 10 =>
  stage := stage + 1;
for i128 in 0 to 7 loop
partsum(i128) <= summator_out(i128);
end loop;
when 11 =>
  stage := stage + 1;
when 12 =>
  stage := stage + 1;
for i133 in 0 to 7 loop
summator_in_a_aux(i133) <= partsum(i133);
end loop;
for i134 in 0 to 7 loop
summator_in_b_aux(i134) <= summator_out(i134);
end loop;
when 13 =>
  stage := stage + 1;
when 14 =>
  stage := stage + 1;
for i139 in 0 to 7 loop
partsum(i139) <= summator_out(i139);
end loop;
when 15 =>
  stage := stage + 1;
when 16 =>
  stage := stage + 1;
when 17 =>
  stage := stage + 1;
when 18 =>
  stage := stage + 1;
for i148 in 0 to 7 loop
summator_in_a_aux(i148) <= partsum(i148);
end loop;
for i149 in 0 to 7 loop
summator_in_b_aux(i149) <= summator_out(i149);
end loop;
when 19 =>
  stage := stage + 1;
when 20 =>
  stage := stage + 1;
when 21 =>
  stage := stage + 1;
when 22 =>
  stage := stage + 1;
when 23 =>
  stage := stage + 1;
when 24 =>
  stage := stage + 1;
for i1 in 0 to 3 loop
summator_in_a_aux(i1) <= summator_out(i1);
summator_in_b_aux(i1) <= summator_out(i1+4);
end loop;
when 25 =>
  stage := stage + 1;
when 26 =>
  stage := stage + 1;
when 27 =>
  stage := stage + 1;
when 28 =>
  stage := stage + 1;
when 29 =>
  stage := stage + 1;
when 30 =>
  stage := stage + 1;
for i1 in 0 to 1 loop
summator_in_a_aux(i1) <= summator_out(i1);
summator_in_b_aux(i1) <= summator_out(i1+2);
end loop;
when 31 =>
  stage := stage + 1;
when 32 =>
  stage := stage + 1;
when 33 =>
  stage := stage + 1;
when 34 =>
  stage := stage + 1;
when 35 =>
  stage := stage + 1;
when 36 =>
  stage := stage + 1;
summator_in_a_aux(0) <= summator_out(0);
summator_in_b_aux(0) <= summator_out(1);
when 37 =>
  stage := stage + 1;
when 38 =>
  stage := stage + 1;
when 39 =>
  stage := stage + 1;
when 40 =>
  stage := stage + 1;
when 41 =>
  stage := stage + 1;
when 42 =>
  stage := stage + 1;
sum <= summator_out(0);
ready <= "00000000000000000000000000000001";
	stage := 0;
when others => null;
    end case;
  end if;
 end if;
end process STATE_PROC;
end IMP;
