



library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v1_00_b;
use proc_common_v1_00_b.proc_common_pkg.all;

entity vector_proc_32 is
 port (
DO: out std_logic_vector(31 downto 0);
ADDR: in std_logic_vector(31 downto 0);
DI: in std_logic_vector(31 downto 0);
EN: in std_logic_vector(0 downto 0);
WE: in std_logic_vector(0 downto 0);
REG_IN_A: in std_logic_vector(31 downto 0);
REG_IN_B: in std_logic_vector(31 downto 0);
REG_OUT_A: out std_logic_vector(31 downto 0);
REG_OUT_B: out std_logic_vector(31 downto 0);
REG_WE_A: in std_logic_vector(0 downto 0);
REG_WE_B: in std_logic_vector(0 downto 0);
Clk: in std_logic;
Reset: in std_logic

);
end vector_proc_32;
architecture IMP of vector_proc_32 is

  component HI_RAMB
   generic (
	data_width: integer;
	address_width: integer;
	depth: integer
	);
   port (
	doa: out std_logic_vector(data_width-1 downto 0);
	dob: out std_logic_vector(data_width-1 downto 0);
	addra: in std_logic_vector(address_width-1 downto 0);
	addrb: in std_logic_vector(address_width-1 downto 0);
	clka: in std_ulogic;
	clkb: in std_ulogic;
	dia: in std_logic_vector(data_width-1 downto 0);
	dib: in std_logic_vector(data_width-1 downto 0);
	rsta: in std_ulogic;
	rstb: in std_ulogic;
	wea: in std_ulogic;
	web: in std_ulogic
	);
  end component;


	signal L: std_logic_vector( 31 downto 0 );
	signal sum: std_logic_vector( 31 downto 0 );
	signal ready: std_logic_vector( 31 downto 0 );
	type l8w32_type is array (0 to 7) of std_logic_vector(31 downto 0);
	type l8w1_type is array (0 to 7) of std_logic_vector(0 downto 0);
	signal array_addra: l8w32_type;
	signal array_addrb: l8w32_type;
	signal array_dina: l8w32_type;
	signal array_doutb: l8w32_type;
	signal array_wea: l8w1_type;
	signal array_dinb: l8w32_type;
	signal array_douta: l8w32_type;
	signal array_web: l8w1_type;
	signal old_addr_low: std_logic_vector( 2 downto 0 );
	signal partsum: l8w32_type;
	signal summator_in_a: l8w32_type;
	signal summator_in_b: l8w32_type;
	signal summator_in_a_aux: l8w32_type;
	signal summator_in_b_aux: l8w32_type;
	signal summator_out: l8w32_type;
	signal enable_sum: std_logic_vector( 0 downto 0 );
	signal enable_head: std_logic_vector( 0 downto 0 );
	signal enable_tail: std_logic_vector( 0 downto 0 );
	signal cycle: std_logic_vector( 3 downto 0 );
	signal saved_first: std_logic_vector( 31 downto 0 );
	signal saved_last: std_logic_vector( 31 downto 0 );

   component floating_add                                                         
	port (                                             
	a             : IN  std_logic_VECTOR(31 downto 0); 
	b             : IN  std_logic_VECTOR(31 downto 0); 
	operation_nd  : IN  std_logic;                                        
	clk           : IN  std_logic;                     
	result        : OUT std_logic_VECTOR(31 downto 0); 
	rdy           : OUT std_logic                      
       );                                                  
   end component;                                          


   component floating_mul
	port (                                             
	a             : IN  std_logic_VECTOR(31 downto 0); 
	b             : IN  std_logic_VECTOR(31 downto 0); 
	operation_nd  : IN  std_logic;                                          
	clk           : IN  std_logic;                     
	result        : OUT std_logic_VECTOR(31 downto 0); 
	rdy           : OUT std_logic                      
       );                                                  
   end component;                                          

	signal mul_a: std_logic_vector( 31 downto 0 );
	signal mul_result: std_logic_vector( 31 downto 0 );

begin

array_gen: for array_i in 0 to 7 generate
begin
	array_inst: HI_RAMB
	generic map (
		 address_width => 32,
		 data_width => 32,
		 depth => 2048
	)
	port map (
		addra => array_addra(array_i),
		addrb => array_addrb(array_i),
		dia => array_dina(array_i),
		wea => array_wea(array_i)(0),
		dob => array_doutb(array_i),
		doa => array_douta(array_i),
		dib => array_dinb(array_i),
		web => array_web(array_i)(0),
		clka => Clk,
		clkb => Clk,
		rsta => Reset,
		rstb => Reset
	);
end generate array_gen;
REG_OUT_A <= sum;
REG_OUT_B <= ready;
loop39_gen: for i1 in 0 to 7 generate
begin
array_dina(i1) <= DI;
array_addra(i1)(28 downto 0) <= ADDR(31 downto 3);
array_addra(i1)(31 downto 29) <= "000";
array_wea(i1) <= "1" when ((i1 = ADDR(2 downto 0)) and (WE = "1")) else "0";
DO <= array_douta(i1) when (i1 = old_addr_low) else (others => 'Z');
end generate loop39_gen;
loop46_gen: for i1 in 0 to 7 generate
begin

   COMPONENT1: floating_add
	port map (
	a => summator_in_a(i1),
	b => summator_in_b(i1),
	result => summator_out(i1),
	operation_nd => enable_sum(0),
	clk => Clk
	 );
summator_in_a(i1) <= "00000000000000000000000000000000" when (enable_head = "1") else
	summator_in_a_aux(i1) when (enable_tail = "1") else
	 summator_out(i1);
summator_in_b(i1) <= summator_in_b_aux(i1) when (enable_tail = "1") else
	 array_doutb(i1);
end generate loop46_gen;

   COMPONENT2: floating_mul
	port map (
	a => mul_a,
	b => "00111111000000000000000000000000",
	result => mul_result,
	operation_nd => '1',
	clk => Clk
	 );

STATE_PROC: process ( Clk ) is
  variable stage: integer := 0;
begin
 if ( Clk'event and Clk = '1' ) then
  if ( Reset = '1' ) then
ready <= "00000000000000000000000000000000";
sum <= "00000000000000000000000000000000";
for i72 in 0 to 7 loop
array_web(i72) <= "0";
end loop;
enable_sum <= "0";
enable_head <= "1";
enable_tail <= "0";
cycle <= "0000";
L <= "00000000000000000000000000000000";
else
old_addr_low <= ADDR(2 downto 0);
    case stage is
when 0 =>
  stage := stage + 1;
if (REG_WE_A = "1") then
L(28 downto 0) <= REG_IN_A(31 downto 3);
for i85 in 0 to 7 loop
array_addrb(i85) <= "00000000000000000000000000000000";
end loop;
ready <= "00000000000000000000000000000000";
sum <= "00000000000000000000000000000000";
for i88 in 0 to 7 loop
array_web(i88) <= "0";
end loop;
enable_sum <= "0";
enable_head <= "1";
enable_tail <= "0";
cycle <= "0000";
else
	stage := 0;
end if;
when 1 =>
  stage := stage + 1;
for i98 in 0 to 7 loop
array_addrb(i98) <= L+"11111111111111111111111111111111";
end loop;
when 2 =>
  stage := stage + 1;
mul_a <= array_doutb(0);
saved_first <= array_doutb(0);
when 3 =>
  stage := stage + 1;
mul_a <= array_doutb(7);
saved_last <= array_doutb(7);
cycle <= "0100";
when 4 =>
  stage := stage + 1;
cycle <= cycle+"1111";
if (cycle /= "0000") then
	stage := 4;
else
array_addrb(0) <= "00000000000000000000000000000000";
array_dinb(0) <= mul_result;
array_web(0) <= "1";
end if;
when 5 =>
  stage := stage + 1;
array_web(0) <= "0";
array_web(7) <= "1";
for i123 in 0 to 7 loop
array_addrb(i123) <= L+"11111111111111111111111111111111";
end loop;
array_dinb(7) <= mul_result;
when 6 =>
  stage := stage + 1;
for i127 in 0 to 7 loop
array_addrb(i127) <= "00000000000000000000000000000000";
end loop;
array_web(7) <= "0";
when 7 =>
  stage := stage + 1;
for i131 in 0 to 7 loop
array_addrb(i131) <= array_addrb(i131)+"00000000000000000000000000000001";
end loop;
L <= L+"11111111111111111111111111111111";
cycle <= "0100";
enable_sum <= "1";
when 8 =>
  stage := stage + 1;
for i138 in 0 to 7 loop
array_addrb(i138) <= array_addrb(i138)+"00000000000000000000000000000001";
end loop;
cycle <= cycle+"1111";
if (cycle = "0000") then
enable_head <= "0";
end if;
L <= L+"11111111111111111111111111111111";
if (L /= "00000000000000000000000000000000") then
	stage := 8;
else
enable_tail <= "1";
end if;
when 9 =>
  stage := stage + 1;
for i151 in 0 to 7 loop
partsum(i151) <= summator_out(i151);
end loop;
when 10 =>
  stage := stage + 1;
for i154 in 0 to 7 loop
summator_in_a_aux(i154) <= partsum(i154);
end loop;
for i155 in 0 to 7 loop
summator_in_b_aux(i155) <= summator_out(i155);
end loop;
when 11 =>
  stage := stage + 1;
for i158 in 0 to 7 loop
partsum(i158) <= summator_out(i158);
end loop;
when 12 =>
  stage := stage + 1;
for i161 in 0 to 7 loop
summator_in_a_aux(i161) <= partsum(i161);
end loop;
for i162 in 0 to 7 loop
summator_in_b_aux(i162) <= summator_out(i162);
end loop;
when 13 =>
  stage := stage + 1;
for i165 in 0 to 7 loop
partsum(i165) <= summator_out(i165);
end loop;
when 14 =>
  stage := stage + 1;
for i168 in 0 to 7 loop
summator_in_a_aux(i168) <= partsum(i168);
end loop;
for i169 in 0 to 7 loop
summator_in_b_aux(i169) <= summator_out(i169);
end loop;
when 15 =>
  stage := stage + 1;
when 16 =>
  stage := stage + 1;
for i174 in 0 to 7 loop
partsum(i174) <= summator_out(i174);
end loop;
when 17 =>
  stage := stage + 1;
when 18 =>
  stage := stage + 1;
for i179 in 0 to 7 loop
summator_in_a_aux(i179) <= partsum(i179);
end loop;
for i180 in 0 to 7 loop
summator_in_b_aux(i180) <= summator_out(i180);
end loop;
when 19 =>
  stage := stage + 1;
when 20 =>
  stage := stage + 1;
for i185 in 0 to 7 loop
partsum(i185) <= summator_out(i185);
end loop;
when 21 =>
  stage := stage + 1;
when 22 =>
  stage := stage + 1;
when 23 =>
  stage := stage + 1;
when 24 =>
  stage := stage + 1;
for i194 in 0 to 7 loop
summator_in_a_aux(i194) <= partsum(i194);
end loop;
for i195 in 0 to 7 loop
summator_in_b_aux(i195) <= summator_out(i195);
end loop;
when 25 =>
  stage := stage + 1;
when 26 =>
  stage := stage + 1;
array_addrb(0) <= "00000000000000000000000000000000";
array_dinb(0) <= saved_first;
array_web(0) <= "1";
when 27 =>
  stage := stage + 1;
array_web(0) <= "0";
array_dinb(7) <= L+"11111111111111111111111111111111";
array_dinb(7) <= saved_last;
array_web(7) <= "1";
when 28 =>
  stage := stage + 1;
array_web(7) <= "0";
when 29 =>
  stage := stage + 1;
when 30 =>
  stage := stage + 1;
for i1 in 0 to 3 loop
summator_in_a_aux(i1) <= summator_out(i1);
summator_in_b_aux(i1) <= summator_out(i1+4);
end loop;
when 31 =>
  stage := stage + 1;
when 32 =>
  stage := stage + 1;
when 33 =>
  stage := stage + 1;
when 34 =>
  stage := stage + 1;
when 35 =>
  stage := stage + 1;
when 36 =>
  stage := stage + 1;
for i1 in 0 to 1 loop
summator_in_a_aux(i1) <= summator_out(i1);
summator_in_b_aux(i1) <= summator_out(i1+2);
end loop;
when 37 =>
  stage := stage + 1;
when 38 =>
  stage := stage + 1;
when 39 =>
  stage := stage + 1;
when 40 =>
  stage := stage + 1;
when 41 =>
  stage := stage + 1;
when 42 =>
  stage := stage + 1;
summator_in_a_aux(0) <= summator_out(0);
summator_in_b_aux(0) <= summator_out(1);
when 43 =>
  stage := stage + 1;
when 44 =>
  stage := stage + 1;
when 45 =>
  stage := stage + 1;
when 46 =>
  stage := stage + 1;
when 47 =>
  stage := stage + 1;
when 48 =>
  stage := stage + 1;
sum <= summator_out(0);
ready <= "00000000000000000000000000000001";
	stage := 0;
when others => null;
    end case;
  end if;
 end if;
end process STATE_PROC;
end IMP;
